package mvd.jester.info;

import java.util.HashSet;
import java.util.Optional;
import java.util.Set;
import mvd.jester.info.TerminationInfo.Level;

/**
 * SchedulingInfo
 */
public class SchedulingInfo {

    private final double parallelTaskRatio;
    private final double utilization;
    private final Set<TerminationInfo> terminationInfos;
    private Optional<TerminationInfo> failedTerminationInfo;

    public SchedulingInfo(double parallelTaskRatio, double utilization) {
        this.parallelTaskRatio = parallelTaskRatio;
        this.utilization = utilization;
        this.terminationInfos = new HashSet<>();
        this.failedTerminationInfo = Optional.empty();
    }

    /**
     * @return the utilization
     */
    public double getUtilization() {
        return utilization;
    }

    /**
     * @return the parallelTaskRatio
     */
    public double getParallelTaskRatio() {
        return parallelTaskRatio;
    }

    public SchedulingInfo(Set<TerminationInfo> terminationInfos, double parallelTaskRatio,
            double utilization) {
        this.terminationInfos = terminationInfos;
        this.parallelTaskRatio = parallelTaskRatio;
        this.utilization = utilization;
        failedTerminationInfo =
                terminationInfos.stream().filter(t -> t.getLateness() > 0).findFirst();
    }

    public boolean checkLevelFail(Level level) {
        return terminationInfos.stream()
                .anyMatch(t -> t.getLateness() > 0 && t.getTaskLevel() == level);
    }

    public boolean checkTasksetFeasible() {
        // return terminationInfos.isEmpty();
        return !terminationInfos.stream().anyMatch(t -> t.getLateness() > 0);
    }

    public boolean addTerminationInfo(TerminationInfo terminationInfo) {
        return terminationInfos.add(terminationInfo);
    }

    /**
     * @return the terminationInfos
     */
    public Set<TerminationInfo> getTerminationInfos() {
        return terminationInfos;
    }

    /**
     * @return the failedTerminationInfo
     */
    public Optional<TerminationInfo> getFailedTerminationInfo() {
        return failedTerminationInfo;
    }

    public void setFailedTerminationInfo(TerminationInfo failedTerminationInfo) {
        this.failedTerminationInfo = Optional.of(failedTerminationInfo);
    }
}
